function [ber_lin,ber_sic] = MU_MIMO_UL_detectors(snr_db)

%  MU-MIMO wireless communication system operating with 
%  spatial multiplexing with linear and SIC (per user) MMSE detectors. 
%  The model describes the uplink of a system in which each 
%  user is equipped with N_U antennas and the access point 
%  has N_A antennas. The network has K users.
%
%  July 2012
%  Authors: Rodrigo C. de Lamare 
%-------------------------------------------------------------------
% Configuration 

% QPSK


% Number of users
K = 3;

% Number of antennas per user
N_U = 2;

% Number of receive antennas
N_A = 8;

% Packet size
N = 100;

% Signal-to-noise ratio (SNR) manipulation
sigma_s = 1; % signal square root power
snr = exp(snr_db*log(10)/10); % conversion from dB  
sigma_n = sqrt( (K*N_U)/snr )*sigma_s; % noise standard deviation

% MMSE filters
W = zeros(N_A,N_U,K); % linear filter for linear detector and K users
W_sic = zeros(N_A,N_U,K); % linear filter for SIC detector and K users

% Counters for errors
ne_lin = 0;
ne_sic = 0;
s_sic = zeros(K*N_U,1);

% Channel Matrix
sigma_c = 1; % channel power standard deviation
H = zeros(N_A,N_U,K);
for k = 1:K
    H(:,:,k) = (sigma_c*randn(N_A,N_U)+sqrt(-1)*(sigma_c*randn(N_A,N_U)))/sqrt(2); 
end
H_sic = H;
% Covariance matrix inversion 
Hc = zeros(N_A,N_A);
Hc_sic = Hc; 
for k = 1:K
    Hc = Hc + H(:,:,k)*H(:,:,k)';
 end

        
R = inv(Hc + (sigma_n^2/sigma_s^2*eye(N_A)));
s_qpsk = zeros(N_U,K);
s = s_qpsk;
z = s;
for l = 1:N
    x = zeros(N_A,1);
    for k = 1:K 
        % QPSK symbols
        s_qpsk(:,k) = [sign(randint(N_U,1)-0.5) + sqrt(-1)*sign(randint(N_U,1)-0.5)];
        % Scaling
        s(:,k) = s_qpsk(:,k)/sqrt(2);
        x = x + H(:,:,k)*s(:,k);
    end
    % Noise    
    n = (sigma_n/sqrt(2))*(randn(N_A,1)+sqrt(-1)*randn(N_A,1));
    % Received vector
    r = x + n;
    % Compute Norms 
    for l = 1:K
          mn(l) = norm(W_sic(:,:,k))^2;
    end
    % Ordering
    [yo,ind] = sort(mn);
    %Hc_sic = zeros(N_A,N_A);
    H_sic = H;
    r_l = r;
    for k = 1:K
        % Allocating the ordering
        j = ind(k);
        % MMSE Linear Detector
        W(:,:,j) = R*H(:,:,j);
        z = W(:,:,j)'*r_l;
        % MMSE SIC Detector  
        W_sic(:,:,j) = W(:,:,j);
        % Filtering
        f = W_sic(:,:,j);
        z_s = f'*r;
        % Slicer for QPSK
        s_sic = sign(real(z_s)) + sqrt(-1)*sign(imag(z_s));
        % Interference Cancellation 
        r = r - H_sic(:,:,j)*s(:,j); %H_sic(:,:,j)*s_sic/sqrt(2); 
        H_sic(:,:,j) = zeros(N_A,N_U); 
        for t = 1:K
            Hc_sic = Hc_sic + H_sic(:,:,t)*H_sic(:,:,t)';
        end
        W_sic(:,:,j) = inv(Hc_sic*Hc_sic' + (sigma_n^2/sigma_s^2*eye(N_A)))*H_sic(:,:,j);
        % Slicer for QPSK
        s_lin = sign(real(z)) + sqrt(-1)*sign(imag(z));
        % Error counting     
        for l = 1:N_U 
            % Linear 
            [ex,ey] = find(real(s_lin(l))~= real(s_qpsk(l,j)));
            ne_lin = ne_lin + length(ex);
            [ex,ey] = find(imag(s_lin(l))~= imag(s_qpsk(l,j)));
            ne_lin = ne_lin + length(ex);
            % SIC
            [ex,ey] = find(real(s_sic(l))~= real(s_qpsk(l,j)));
            ne_sic = ne_sic + length(ex);
            [ex,ey] = find(imag(s_sic(l))~= imag(s_qpsk(l,j)));
            ne_sic = ne_sic + length(ex);
        end
    end
 end
ber_lin = ne_lin/(2*N*K*N_U);
ber_sic = ne_sic/(2*N*K*N_U);